indexing

	description: "Generates XHTML."

	todo:
		"1. should use binary search allowed_new_line_tag"

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #2 $"

class

	EPX_XHTML_WRITER

inherit

	EPX_XML_WRITER
		rename
			xml as document
		redefine
			empty_tag_closing_chars,
			indent,
			new_line_after_closing_tag,
			new_line_before_starting_tag
		end


creation

	make,
	make_with_capacity,
	make_fragment,
	make_fragment_with_capacity


feature -- overrule some xml stuff

	new_line_after_closing_tag (a_tag: STRING) is
		do
			if allowed_new_line_tag (a_tag) then
				precursor (a_tag)
			end
		end

	new_line_before_starting_tag (a_tag: STRING) is
		do
			if allowed_new_line_tag (a_tag) then
				precursor (a_tag)
			end
		end


feature {NONE} -- overrule some xml layout

	allowed_new_line_tag (a_tag: STRING): BOOLEAN is
		do
			Result :=
				a_tag.is_equal ("p") or else
				a_tag.is_equal ("br") or else
				a_tag.is_equal ("td") or else
				a_tag.is_equal ("tr") or else
				a_tag.is_equal ("table") or else
				a_tag.is_equal ("select") or else
				a_tag.is_equal ("option") or else
				a_tag.is_equal ("pre") or else
				a_tag.is_equal ("h3") or else
				a_tag.is_equal ("h2") or else
				a_tag.is_equal ("h1") or else
				a_tag.is_equal ("form") or else
				a_tag.is_equal ("html") or else
				a_tag.is_equal ("meta") or else
				a_tag.is_equal ("head") or else
				a_tag.is_equal ("body") or else
				a_tag.is_equal ("title")
			end

			empty_tag_closing_chars: STRING is
			-- XHTML applications like an additional space
		once
			Result := " />"
		end

	indent is
			-- somewhat less indenting
		local
			spaces: STRING
		do
			if tags.count > 2 then
				spaces := sh.make_spaces (2 * (tags.count - 2 ) )
				extend (spaces)
			end
		end


feature -- doctype

	doctype is
		do
			doctype_strict
		end

	doctype_frameset is
			-- Output will be frame-based
		do
			add_header_iso_8859_1_encoding
			extend ("<!DOCTYPE html PUBLIC %"-//W3C//DTD XHTML 1.0 Frameset//EN%"%
					  % %"DTD/xhtml1-frameset.dtd%">")
			new_line
		end

	doctype_strict is
			-- Output will be strict XHTML
		do
			add_header_iso_8859_1_encoding
			extend ("<!DOCTYPE html PUBLIC %"-//W3C//DTD XHTML 1.0 Strict//EN%"%
					  % %"DTD/xhtml1-strict.dtd%">")
			new_line
		end

	doctype_transitional is
			-- Output will be transitional XHTML
		do
			add_header_iso_8859_1_encoding
			extend ("<!DOCTYPE html PUBLIC %"-//W3C//DTD XHTML 1.0 Transitional//EN%"%
					  % %"DTD/xhtml1-transitional.dtd%">")
			new_line
		end


feature -- page

	b_html is
			-- start html page
		do
			start_tag ("html")
			set_attribute ("xmlns", "http://www.w3.org/1999/xhtml")
			set_ns_attribute ("xml:", "lang", "en")
			set_attribute ("lang", "en")
		end

	e_html is
		require
			valid_stop: is_started ("html")
		do
			stop_tag
		end


feature -- header

	meta_refresh_other (time: INTEGER; url: STRING) is
		do
			start_tag ("meta")
			set_attribute ("http-equiv", "refresh")
			set_attribute ("content", time.out + "; URL=" + url)
			stop_tag
		end

	b_head is
		do
			start_tag ("head")
		end

	e_head is
		require
			valid_stop: is_started ("head")
		do
			stop_tag
		end

	title (a_text: STRING) is
		require
			head_started: is_started ("head")
		do
			add_tag ("title", a_text)
		end


feature -- body

	b_body is
		do
			start_tag ("body")
		end

	e_body is
		require
			valid_stop: is_started ("body")
		do
			stop_tag
		end


feature --section headers

	h1 (header_text: STRING) is
		do
			add_tag ("h1", header_text)
		end


feature -- paragraph

	br is
			-- break, the Appendix C.3 way
		do
			start_tag ("br")
			stop_tag
		end

	b_p is
		do
			start_tag ("p")
		end

	e_p is
		require
			valid_stop: is_started ("p")
		do
			stop_tag
		end

	p (par: STRING) is
		do
			add_tag ("p", par)
		end


feature -- layout

	b_tt is
			-- teletype writer font
		do
			start_tag ("tt")
		end

	e_tt is
		require
			valid_stop: is_started ("tt")
		do
			stop_tag
		end


feature -- link

	b_a (href: STRING) is
		require
			valid_containment: not is_a_parent ("a")
		do
			start_tag ("a")
			set_attribute ("href", href)
		end

	e_a is
		require
			valid_stop: is_started ("a")
		do
			stop_tag
		end

	a (href, s: STRING) is
		do
			b_a (href)
			puts (s)
			e_a
		end



feature -- rules

	hr is
			-- horizontal rule
		do
			start_tag ("hr")
			stop_tag
		end


feature -- white space

	nbsp is
			-- non breaking white space
		do
			add_entity ("nbsp")
		end


feature -- verbatim

	b_pre is
		do
			start_tag ("pre")
			put_new_line
		end

	e_pre is
		require
			valid_stop: is_started ("pre")
		do
			-- make sure </pre> always starts on its own line
			if not on_new_line then
				new_line
			end
			stop_tag
		end


feature -- tables

	b_table is
		do
			start_tag ("table")
		end

	e_table is
		require
			valid_stop: is_started ("table")
		do
			stop_tag
		end

	b_tr is
		require
			table_started: is_started ("table")
		do
			start_tag ("tr")
		end

	e_tr is
		require
			valid_stop: is_started ("tr")
		do
			stop_tag
		end

	td is
			-- an empty cell
		do
			b_td
			e_td
		end

	b_td is
		require
			row_started: is_started ("tr")
		do
			start_tag ("td")
		end

	e_td is
		require
			valid_stop: is_started ("td")
		do
			stop_tag
		end

	th is
			-- an empty cell
		do
			b_th
			e_th
		end

	b_th is
		require
			row_started: is_started ("tr")
		do
			start_tag ("th")
		end

	e_th is
		require
			valid_stop: is_started ("th")
		do
			stop_tag
		end


feature -- forms

	standard_encoding: STRING is "application/x-www-form-urlencoded"

	plaintext_encoding: STRING is "text/plain"

	multipart_encoding: STRING is "multipart/form-data"

	b_form (method, action: STRING) is
		require
			valid_method: method /= Void and then not method.is_empty
			valid_action: action /= Void and then not action.is_empty
		do
			start_tag ("form")
			set_attribute ("method", method)
			set_attribute ("action", action)
		end

	b_form_get (action: STRING) is
		do
			b_form ("GET", action)
		end

	b_form_post (action: STRING) is
		do
			b_form ("POST", action)
		end

	e_form is
		require
			valid_stop: is_started ("form")
		do
			stop_tag
		end

	b_input (type, name: STRING) is
		require
			form_started: is_a_parent ("form")
			type_not_empty: type /= Void and then not type.is_empty
			valid_type:
				type.is_equal ("button") or else
				type.is_equal ("checkbox") or else
				type.is_equal ("file") or else
				type.is_equal ("hidden") or else
				type.is_equal ("image") or else
				type.is_equal ("password") or else
				type.is_equal ("radio") or else
				type.is_equal ("reset") or else
				type.is_equal ("submit") or else
				type.is_equal ("text")
				name_not_empty: name /= Void and then not name.is_empty
		do
			start_tag ("input")
			set_attribute ("type", type)
			set_attribute ("name", name)
		end

	e_input is
		require
			valid_stop: is_started ("input")
		do
			stop_tag
		end

	button_hidden (name, value: STRING) is
		do
			b_input ("hidden", name)
			set_attribute ("value", value)
			e_input
		end

	b_button_submit (name, value: STRING) is
		do
			b_input ("submit", name)
			set_attribute ("value", value)
		end

	e_button_submit is
		do
			e_input
		end

	b_button_reset is
		do
			start_tag ("input")
			set_attribute ("type", "reset")
		end

	e_button_reset is
		do
			e_input
		end

	button_reset is
		do
			b_button_reset
			e_button_reset
		end

	b_checkbox (name, value: STRING) is
		require
			have_value: value /= Void and not value.is_empty
		do
			b_input ("checkbox", name)
			set_attribute ("value", value)
		end

	e_checkbox is
		do
			e_input
		end

	b_radio (name, value: STRING) is
		require
			have_value: value /= Void and not value.is_empty
		do
			b_input ("radio", name)
			set_attribute ("value", value)
		end

	e_radio is
		do
			e_input
		end

	b_select (name: STRING) is
		do
			start_tag ("select")
			set_attribute ("name", name)
			end

			e_select is
		require
			valid_stop: is_started ("select")
		do
			stop_tag
		end

	option (text: STRING) is
		do
			add_tag ("option", text)
		end

	selected_option (choice: STRING) is
		do
			start_tag ("option")
			set_attribute ("selected", Void)
			puts (choice)
			stop_tag
		end

	b_textarea (name: STRING) is
			-- multiline input
		do
			start_tag ("textarea")
			set_attribute ("name", name)
		end

	e_textarea is
		require
			valid_stop: is_started ("textarea")
		do
			stop_tag
		end

	input_text (name: STRING; size: INTEGER; value: STRING) is
			-- singleline input
		do
			b_input ("text", name)
			set_attribute ("size", size.out)
			set_attribute ("value", value)
			e_input
		end


feature -- CSS style sheet support

	b_style is
		do
			start_tag ("style")
			set_attribute ("type", "text/css")
		end

	e_style is
		require
			valid_stop: is_started ("style")
		do
			stop_tag
		end

	set_class (name: STRING) is
			-- set attribute class
		require
			valid_name: name /= Void and then not name.is_empty
		do
			set_attribute ("class", name)
		end


end
