indexing

	description: "Class that implements the basics of an HTTP 1.1 client."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"


deferred class

	EPX_HTTP_CLIENT

inherit

	SUS_BASE

feature -- Creation

	make (host_name: STRING) is
			-- Prepare for request to `host_name'.
		require
			host_name_not_empty: host_name /= Void and then not host_name.is_empty
		do
			make_from_port (host_name, 0)
		end

	make_from_port (host_name: STRING; port: INTEGER) is
			-- Prepare for request.
			-- Use `port' is 0 to use the default port (80).
		require
			host_name_not_empty: host_name /= Void and then not host_name.is_empty
			valid_port: port >= 0 and port <= 65535
		do
			create host.make_from_name (host_name)
			make_from_host_and_port (host, port)
		end

	make_from_host (a_host: SUS_HOST) is
			-- Prepare for request to resolved `a_host'. If `port' is 0,
			-- the default port is taken, else the port can be overruled.
		require
			host_found: a_host /= Void and then a_host.found
		do
			make_from_host_and_port (a_host, 0)
		end

	make_from_host_and_port (a_host: SUS_HOST; port: INTEGER) is
			-- Prepare for request to `a_host'. If `port' is 0, the
			-- default port is taken, else the port can be overruled.
		require
			host_found: a_host /= Void and then a_host.found
			valid_port: port >= 0 and port <= 65535
		local
			service: SUS_SERVICE
		do
			host := a_host
			if port = 0 then
				service := http_service
			else
				create service.make_from_port (port, "tcp")
			end
			create sa.make (host, service)
		end


feature -- Requests

	get (path: STRING) is
		require
			path_not_empty: path /= Void and then not path.is_empty
		deferred
		end


feature -- Response

	read_response is
			-- Read entire resonse.
		deferred
		end


feature -- Client http version

	client_version: STRING is
			-- Client's version of the http protocol
		deferred
		ensure
			have_version: Result /= Void
		end


feature -- Public state

	http: SUS_TCP_SOCKET
			-- Our socket connection, opened after request has been sent.


feature {NONE} -- Private state

	host: SUS_HOST
			-- Host where requests are send to.

	sa: SUS_SOCKET_ADDRESS
			-- Binding of host to http port.

	http_service: SUS_SERVICE is
			-- Default http port.
		once
			create Result.make_from_name ("http", "tcp")
		ensure
			service_found: Result /= Void
		end

feature {NONE} -- Implementation

	assert_http_closed is
			-- Make sure http is no longer connected.
		do
			if http /= Void and then http.is_open then
				http.close
			end
		ensure
			http_closed: http = Void or else http.is_closed
		end


invariant

	host_found: host /= Void and then host.found
	have_address: sa /= Void

end
