indexing

	description: "Base class to converts arrays to pointers."

	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #2 $"


deferred class

	ABSTRACT_ARRAY_HELPER

inherit

	EPX_EXTERNAL_HELPER

	EPX_POINTER_ADD


feature

	string_array_to_pointer_array (a: ARRAY [STRING]): ARRAY [POINTER] is
			-- convert ARRAY[STRING] to char ** (ARRAY[POINTER])
			-- resulting array is null terminated
			-- result of this can be passed to `pointer_array_to_pointer'
		local
			i: INTEGER
			s: STRING
		do
			-- create new array with room for null terminator
			if a.item (a.upper) = Void then
				create Result.make (a.lower, a.upper)
			else
				create Result.make (a.lower, a.upper+1)
			end

			-- copy the strings in the new array as pointers
			from
				i := a.lower
			until
				i > a.upper
			loop
				s := a.item (i)
				if s = Void then
					Result.put (default_pointer, i)
				else
					Result.put (sh.string_to_pointer(s), i)
				end
				i := i + 1
			end
			Result.put (default_pointer, Result.upper)
		ensure
			null_terminated: Result.item (Result.upper) = default_pointer
		end

	pointer_to_string_array (p: POINTER): ARRAY [STRING] is
			-- convert char ** to ARRAY[STRING]
			-- returns an empty array if p = default_pointer
		local
			ptr1, ptr2: POINTER
			s: STRING
		do
			create Result.make (0, -1)

			if p /= default_pointer then
				ptr1 := p
				from
					ptr2 := posix_pointer_contents (ptr1)
				until
					ptr2 = default_pointer
				loop
					s := sh.pointer_to_string (ptr2)
					Result.force (s, Result.upper+1)
					ptr1 := posix_pointer_advance (ptr1)
					ptr2 := posix_pointer_contents (ptr1)
				end
			end
		ensure
			valid_result: Result /= Void
		end


feature -- compiler specific conversions

	integer_array_to_pointer (a: ARRAY [INTEGER]): POINTER is
			-- convert to int ** (??)
		require
			valid_array: a /= Void
		deferred
		ensure
			valid_result: Result /= default_pointer
		end

	pointer_array_to_pointer (a: ARRAY [POINTER]): POINTER is
			-- convert to void **
		require
			valid_array: a /= Void
		deferred
		ensure
			valid_result: Result /= default_pointer
		end


end
