indexing

	description: "Class that describes a host on the network."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"


class

	SUS_HOST


inherit

	SUS_BASE

	SAPI_NETDB

	EPX_POINTER_ADD


creation

	make_from_name,
	make_from_address


feature -- creation

	make_from_name (a_name: STRING) is
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
		do
			name := a_name
			find_by_name
		end

	make_from_address (a_address: SUS_IP_ADDRESS) is
		require
			valid_address: a_address /= Void
		do
			-- find_by_address
		end


feature -- Command

	find_by_name is
			-- Attempt to lookup up the host given in `name'. Sets
			-- `found' if host could be found.
			-- If found, sets `canonical_name', `aliases',
			-- `address_type', `address_length' and `addresses'.
		require
			name_not_empty: name /= Void and then not name.is_empty
		local
			p: POINTER
			pptr: POINTER
			aptr: POINTER
			ip_address: SUS_IP4_ADDRESS
			buf: STDC_BUFFER
		do
			p := posix_gethostbyname (sh.string_to_pointer (name))
			if p = default_pointer then
				canonical_name := Void
				aliases := Void
				addresses := Void
			else
				canonical_name := sh.pointer_to_string (posix_hostent_h_name (p))
				aliases := ah.pointer_to_string_array (posix_hostent_h_aliases (p))
				address_type := posix_hostent_h_addrtype (p)
				address_length := posix_hostent_h_length (p)
				from
					create addresses.make (0, -1)
					pptr := posix_hostent_h_addr_list (p)
					aptr := posix_pointer_contents (pptr)
				until
					aptr = default_pointer
				loop
					create buf.make_from_pointer (aptr, 4, False)
					create ip_address.make_from_integer (buf.peek_uint32_native (0))
					addresses.force (ip_address, addresses.upper + 1)
					pptr := posix_pointer_advance (pptr)
					aptr := posix_pointer_contents (pptr)
				end
			end
		end


feature -- Queries

	found: BOOLEAN is
			-- True if `name' was found.
		do
			Result := addresses /= Void
		end

	name: STRING
			-- Name as given to `make_from_name' or else equal to
			-- `canonical_name'.

	canonical_name: STRING
			-- Official (canonical) name of host.

	aliases: ARRAY[STRING]
			-- Alias names.

	address_type: INTEGER
			-- Host address type: AF_INET or AF_INET6

	address_length: INTEGER
			-- Length of address: 4 or 16.

	addresses: ARRAY[SUS_IP_ADDRESS]
			-- Array with IPv4 or IPv6 addresses.


invariant

	name_not_empty: name /= Void and then not name.is_empty
	has_canonical_name: found = (canonical_name /= Void)
	has_at_least_one_ip_address: found = (addresses /= Void and then addresses.count > 0)
	has_aliases: found = (aliases /= Void)
	valid_length: found implies address_length > 0
	consistent: addresses /= Void and then addresses.count > 0 implies found

end
