indexing

	description: "Class that covers the Single Unix Spec timeval structure."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #1 $"


class

	SUS_TIME_VALUE


inherit

	SUS_BASE

	SAPI_TIME

creation

	make,
	make_from_now

feature {NONE} -- Initialization

	make is
		do
			if tp = Void then
				create tp.allocate_and_clear (posix_timeval_size)
			end
		end

feature -- Initialization

	make_from_now is
		do
			make
			safe_call (posix_gettimeofday (tp.ptr))
		end

feature -- Queries

	seconds: INTEGER is
		-- Seconds since 0:00, 1970 UTC
		do
			Result := posix_timeval_tv_sec (tp.ptr)
		ensure
			positive: Result >= 0
		end

	microseconds: INTEGER is
			-- Microseconds elapsed since `seconds'
		do
			Result := posix_timeval_tv_usec (tp.ptr)
		ensure
			in_range: Result >= 0 and Result <= 999999
		end

feature {NONE} -- Private state

	tp: STDC_BUFFER

invariant

	have_tp: tp /= Void and then tp.capacity = posix_timeval_size
	seconds_positive: seconds >= 0
	microseconds_valid: microseconds >= 0 and microseconds <= 999999

end
