indexing

	description: "Windows child processes created by call to CreateProcess."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"


class

	EPX_EXEC_PROCESS


inherit

	ABSTRACT_EXEC_PROCESS

	WINDOWS_PROCESS
		redefine
			terminate
		end

	WINDOWS_BASE


creation

	make,
	make_capture_input,
	make_capture_output,
	make_capture_io,
	make_capture_all


feature -- child's pid

	pid: INTEGER is
		do
			Result := procinfo.pid
		end

	is_pid_valid: BOOLEAN is
			-- Does Current refer to an actual child process?
		do
			Result := procinfo /= Void
		end


feature -- execute

	execute is
			-- Executes `program_name'.
			-- Don't forget to `wait' for this process to terminate.
		local
			si: WINDOWS_STARTUPINFO
			command_line: STRING
			sa: WINDOWS_SECURITY_ATTRIBUTES
		do
			-- reset some state
			error_pipe := Void
			input_pipe := Void
			output_pipe := Void

			command_line := program_name + all_arguments
			create si.make
			if capture_input or else capture_output or else capture_error then
				si.set_dwFlags (STARTF_USESHOWWINDOW + STARTF_USESTDHANDLES)
			else
				si.set_dwFlags (STARTF_USESHOWWINDOW)
			end
			create sa.make
			sa.set_inherit_handle (True)
			if capture_input then
				create input_pipe.make_inheritable_write
				si.set_hStdInput (input_pipe.fdout.value)
			else
				si.set_hStdInput (posix_getstdhandle (STD_INPUT_HANDLE))
			end
			if capture_output then
				create output_pipe.make_inheritable_read
				si.set_hStdOutput (output_pipe.fdin.value)
			else
				si.set_hStdOutput (posix_getstdhandle (STD_OUTPUT_HANDLE))
			end
			if capture_error then
				create error_pipe.make_inheritable_read
				si.set_hStdError (error_pipe.fdin.value)
			else
				si.set_hStdError (posix_getstdhandle (STD_ERROR_HANDLE))
			end
			si.set_wShowWindow (SW_HIDE)
			create procinfo.make
			safe_win_call (
				posix_createprocess (
					default_pointer,
					sh.string_to_pointer (command_line),
					default_pointer,
					default_pointer,
					True,
					0,
					default_pointer,
					sh.string_to_pointer (initial_directory),
					si.ptr,
					procinfo.ptr))
			running := True
			-- close any unnecessary (??) handles.
			safe_win_call (posix_closehandle (procinfo.thread_handle))

			-- child has handles now, close our part of the handle
			if capture_input then
				fd_stdin := input_pipe.fdin
				input_pipe.fdout.close
			else
				fd_stdin := Void
			end
			if capture_output then
				fd_stdout := output_pipe.fdout
				output_pipe.fdin.close
			else
				fd_stdout := Void
			end
			if capture_error then
				fd_stderr := error_pipe.fdout
				error_pipe.fdin.close
			else
				fd_stderr := Void
			end
		end


feature -- Signal this process

	terminate is
			-- This is definitively not a graceful termination.
		do
			safe_win_call (posix_terminateprocess (procinfo.process_handle, Terminate_exit_code))
		end


feature -- Actions that parent may execute

	wait_for (suspend: BOOLEAN) is
			-- not `suspend' is not yet supported.
		local
			dw: INTEGER
			success: BOOLEAN
		do
			if running then
				dw := posix_waitforsingleobject (procinfo.process_handle, INFINITE)
				success := dw /= WAIT_FAILED
				if not success then
					raise_windows_error
				end
				safe_win_call (posix_getexitcodeprocess (procinfo.process_handle, $exit_code))
				-- already closed:
				-- safe_win_call (posix_closehandle (procinfo.thread_handle))
				safe_win_call (posix_closehandle (procinfo.process_handle))
				if input_pipe /= Void then
					input_pipe.close
					input_pipe := Void
				end
				if output_pipe /= Void then
					output_pipe.close
					output_pipe := Void
				end
				if error_pipe /= Void then
					error_pipe.close
					error_pipe := Void
				end
				running := False
			end
		end


feature -- Termination info

	exit_code: INTEGER


feature {NONE} -- Implementation

	all_arguments: STRING is
			-- return arguments as a single string
			-- first character returned is a space
		local
			i: INTEGER
		do
			create Result.make (0)
			from
				i := arguments.lower
			until
				i > arguments.upper
			loop
				Result := Result + " " + arguments.item (i)
				i := i + 1
			end
		end


feature {NONE} -- Private state

	procinfo: WINDOWS_PROCESS_INFORMATION

	initial_directory: STRING is
		do
			Result := Void
		end

	input_pipe,
	output_pipe,
	error_pipe: WINDOWS_PIPE


end
