indexing

   description: "Test epx factory classes."

   author: "Berend de Boer"
   date: "$Date: 2003/01/09 $"
   revision: "$Revision: #2 $"

class 
   
   TEST_EPX_FACTORY
   
   
inherit
   
   EPX_FACTORY
   

creation
   
   make
   

feature

   make is
      do
         print ("Test epx factory (obsolete).%N")
         
--           test_file_system
         
--           test_directory_browsing
         
         test_file_descriptor
         
--           test_binary_mode
         
--           test_set_binary_mode

         print  ("End of test epx factory.%N")
      end
   

feature -- tests   

   test_file_system is
      local
         dir: STRING
      do
         print ("Test directory functions%N")
         dir := fs.getcwd
         fs.chdir ("..")
         print ("Dir: ")
         print (fs.getcwd)
         print (".%N")
         fs.chdir (dir)
         print ("Dir: ")
         print (fs.getcwd)
         print (".%N")
         if not equal(dir, fs.getcwd) then
            print ("!! Directory is not the expected directory.%N")
         end
         if fs.is_directory ("abc") then
            fs.remove_directory ("abc")
         end
            
         fs.mkdir ("abc")
         
         if not fs.status ("abc").is_directory then
            print ("!! status returned that abc isn't a directory.%N")
         end
         
         if not fs.status ("abc/").is_directory then
            print ("!! status returned that abc isn't a directory.%N")
         end
         
         print ("device number of abc: ")
         print (fs.status ("abc").device_number)
         print ("%N")
         
         fs.chdir ("abc")
         print ("Dir: ")
         print (fs.current_directory)
         print (".%N")
         fs.change_directory ("..")
         fs.rename_to ("abc", "def")
         fs.force_remove_directory ("def")
         
         print ("path separator: ")
         print (fs.path_separator)
         print ("%N")
         if fs.is_case_sensitive then
            print ("File system is case-sensitive.%N")
         else
            print ("File system is not case-sensitive.%N")
         end
      end
   
   
   test_directory_browsing is
      local
         i: INTEGER
         dir: ABSTRACT_DIRECTORY
         b: BOOLEAN
         full_name: STRING
         filter: EXTENSION_FILTER
      do
         print ("Test directory reading.%N")
         from
            dir := fs.browse_directory ("src")
            dir.set_recursive (True)
            create filter.make (".e")
            dir.set_filter (filter)
            dir.start
            i := 0
         until
            dir.exhausted
         loop
            i := i + 1
            full_name := dir.full_name
            print (full_name)
            if fs.status (full_name).is_directory then
               print ('/')
            end
            print (" (")
            print (fs.status (full_name).size)
            print (")")
            print ("%N")
            b := fs.is_readable (full_name)
            b := fs.is_writable (full_name)
            dir.forth
         end
         dir.close
         print ("Number of entries: ")
         print (i)
         print ("%N")         
      end
   
   test_file_descriptor is
      local
         fd: ABSTRACT_FILE_DESCRIPTOR
         buf: STDC_BUFFER
         byte: INTEGER
      do
         fd := create_read_write ("test.berend")

         print ("Write a byte to that file.%N")
         create buf.allocate (2)
         buf.poke_uint8 (0, 65)
         buf.poke_uint8 (1, 66)
         fd.write_buffer (buf, 0, buf.capacity)
         fd.close
         
         print ("Open existing file with read-only access.%N")
         fd := open_read ("test.berend")

         print ("Read a byte.%N")         
         fd.read_buffer (buf, 0, buf.capacity)
         byte := buf.peek_uint8 (0)
         if byte = 65 then
            print ("Result is as expected.%N")
         else
            print ("!! Read: ")
            print (byte.out)
            print (" instead of 65.%N")
         end
         
         if fd.status.size /= 2 then
            print ("!! Expected file size 2, got file size: ")
            print (fd.status.size)
            print ("%N")
         end
         if not fd.status.is_regular_file then
            print ("!! is_regular_file returned False.%N")
         end
         if fd.status.is_directory then
            print ("!! is_directory returned True.%N")
         end
         if fd.status.is_fifo then
            print ("!! is_fifo returned True.%N")
         end
         if fd.status.number_of_hard_links /= 1 then
            print ("!! number of hard links not equal to one.%N")
         end
         
         -- test seek
         create buf.allocate_and_clear (2)
         fd.seek (0)
         fd.read_buffer (buf, 0, buf.capacity)
         byte := buf.peek_uint8 (0)
         if byte /= 65 then
            print ("!! seek failed. Read: ")
            print (byte.out)
            print (" instead of 65.%N")
         end
         
         print ("File information.%N")         
         print ("device: ")         
         print (fd.status.device_number.out)
         print (".%N")
         print ("size: ")         
         print (fd.status.size)
         print (".%N")
         print ("access_time: ")         
         print (fd.status.access_time)
         print (".%N")
         print ("change_time: ")         
         print (fd.status.change_time)
         print (".%N")
         print ("mtime: ")         
         print (fd.status.mtime.out)
         print (".%N")
         print ("fs.status.change_time: ")
         print (fs.status ("test.berend").change_time)
         print (".%N")
         
         print ("Test isatty.%N")
         if fd.is_attached_to_terminal then
            print ("Is attached to terminal.%N")
         else
            print ("Is not attached to terminal.%N")
         end
         
         fd.close
         
         fs.remove_file ("test.berend")
      end
   
   test_binary_mode is
      local
         size: INTEGER
         filename: STRING
         buf: STDC_BUFFER
         from_file,
         to_file: ABSTRACT_FILE_DESCRIPTOR
      do         
         filename := "chicken.bmp"
         if fs.is_readable (filename) then
            print ("Test binary file copying.%N")
            size := fs.status (filename).size
            create buf.allocate (size)
            from_file := open_read (filename)
            to_file := create_write (filename + ".copy")
            from_file.read_buffer (buf, 0, size)
            to_file.write_buffer (buf, 0, size)
            if from_file.status.size /= to_file.status.size then
               print ("!! size differs.%N")
            end
            to_file.close
            from_file.close
         end
      end
   
   test_set_binary_mode is
         -- change binary mode
      local
         size: INTEGER
         filename: STRING
         buf: STDC_BUFFER
         from_file,
         to_file: ABSTRACT_FILE_DESCRIPTOR
      do         
         filename := "chicken.bmp"
         if fs.is_readable (filename) then
            print ("Test binary file copying.%N")
            size := fs.status (filename).size
            print ("file size: ")
            print (size)
            print ("%N")
            create buf.allocate (size)
            from_file := open_read (filename)
            to_file := create_write (filename + ".copy")
            from_file.read_buffer (buf, 0, size)
            to_file.write_buffer (buf, 0, size)
            if from_file.status.size /= to_file.status.size then
               print ("!! size differs.%N")
            end
            to_file.close
            from_file.close
         end
      end


end
